@@ -1,28 +1,25 @@
-# -*- coding: utf-8 -*-
 '''
 This module implements a :class:`skosprovider.providers.VocabularyProvider`
 for http://www.heritagedata.org.
 '''
 
-import requests
-from requests.exceptions import ConnectionError
-
-import warnings
 import logging
-log = logging.getLogger(__name__)
+import warnings
 
+import requests
 from language_tags import tags
 from rdflib.namespace import SKOS
-
+from requests.exceptions import ConnectionError
 from skosprovider.exceptions import ProviderUnavailableException
 from skosprovider.providers import VocabularyProvider
 
-from skosprovider_heritagedata.utils import (
-    _split_uri, 
-    uri_to_graph,
-    conceptscheme_from_uri,
-    things_from_graph
-)
+from skosprovider_heritagedata.utils import _split_uri
+from skosprovider_heritagedata.utils import conceptscheme_from_uri
+from skosprovider_heritagedata.utils import things_from_graph
+from skosprovider_heritagedata.utils import uri_to_graph
+
+log = logging.getLogger(__name__)
+
 
 class HeritagedataProvider(VocabularyProvider):
     """A provider that can work with the Heritagedata services of
@@ -42,6 +39,9 @@ class HeritagedataProvider(VocabularyProvider):
         """
         if not 'default_language' in metadata:
             metadata['default_language'] = 'en'
+        if not 'subject' in metadata:
+            metadata['subject'] = []
+        self.metadata = metadata
         if 'scheme_uri' in kwargs:
             self.base_scheme_uri = _split_uri(kwargs['scheme_uri'], 0)
             self.scheme_id = _split_uri(kwargs['scheme_uri'], 1)
@@ -54,10 +54,21 @@ class HeritagedataProvider(VocabularyProvider):
             self.service_scheme_uri = kwargs['service_scheme_uri'].strip('/')
         else:
             self.service_scheme_uri = "http://heritagedata.org/live/services"
-        concept_scheme = conceptscheme_from_uri(self.scheme_uri)
-        super(HeritagedataProvider, self).__init__(metadata, concept_scheme=concept_scheme, **kwargs)
+        self.session = kwargs.get('session', requests.Session())
+
+    @property
+    def concept_scheme(self):
+        return self._get_concept_scheme()
+
+    def _get_concept_scheme(self):
+        return conceptscheme_from_uri(
+            self.scheme_uri,
+            session=self.session
+        )
 
     def _get_language(self, **kwargs):
+        if 'language' in kwargs:
+            return kwargs['language']
         return self.metadata['default_language']
 
     def get_by_id(self, id):
@@ -65,15 +76,18 @@ class HeritagedataProvider(VocabularyProvider):
 
         :param (str) id: integer id of the :class:`skosprovider.skos.Concept` or :class:`skosprovider.skos.Concept`
         :return: corresponding :class:`skosprovider.skos.Concept` or :class:`skosprovider.skos.Concept`.
-            Returns None if non-existing id
+            Returns False if non-existing id
         """
-        graph = uri_to_graph('%s/%s/%s.rdf' % (self.scheme_uri, "concepts", id))
+        graph = uri_to_graph(
+            '{}/{}/{}.rdf'.format(self.scheme_uri, "concepts", id),
+            session=self.session
+        )
         if graph is False:
             return False
         # get the concept
         things = things_from_graph(graph, self.concept_scheme)
         if len(things) == 0:
-            return None
+            return False
         c = things[0]
         return c
 
@@ -82,12 +96,12 @@ class HeritagedataProvider(VocabularyProvider):
 
         :param (str) uri: string uri of the :class:`skosprovider.skos.Concept` or :class:`skosprovider.skos.Concept`
         :return: corresponding :class:`skosprovider.skos.Concept` or :class:`skosprovider.skos.Concept`.
-            Returns None if non-existing id
+            Returns False if non-existing id
         """
         id = _split_uri(uri, 1)
         return self.get_by_id(id)
 
-    def find(self, query):
+    def find(self, query, **kwargs):
         '''Find concepts that match a certain query.
 
         Currently query is expected to be a dict, so that complex queries can
@@ -95,6 +109,16 @@ class HeritagedataProvider(VocabularyProvider):
         with a certain label, with a certain type and for concepts that belong
         to a certain collection.
 
+        .. warning::
+
+            The underlying service returns labels without specifying if they are
+            prefLabels or altLabels. For a certain concept several labels are
+            returned. This method does not return labels, but returns concepts.
+            When multiple labels are detected for a single concept, only one
+            label is attached to this concept. Since no information is present
+            about the type of this label, this can be an altLabel for a concept
+            where a prefLabel exists.
+
         .. code-block:: python
 
             # Find anything that has a label of church.
@@ -163,9 +187,13 @@ class HeritagedataProvider(VocabularyProvider):
             warnings.warn("This provider doesn't support collections at the moment of implementation because Heritagedata doesn't use SKOS:Collection.", UserWarning)
             raise ValueError('You are searching for items in an unexisting collection.')
         params = {'schemeURI': self.scheme_uri, 'contains': label}
-        return self._get_items("getConceptLabelMatch", params)
+        ret = self._get_items("getConceptLabelMatch", params, **kwargs)
+        language = self._get_language(**kwargs)
+        sort = self._get_sort(**kwargs)
+        sort_order = self._get_sort_order(**kwargs)
+        return self._sort(ret, sort, language, sort_order == 'desc')
 
-    def get_all(self):
+    def get_all(self, **kwargs):
         """
         Not supported: This provider does not support this. The amount of results is too large
         """
@@ -175,29 +203,37 @@ class HeritagedataProvider(VocabularyProvider):
         )
         return False
 
-    def get_top_concepts(self):
+    def get_top_concepts(self, **kwargs):
         """  Returns all concepts that form the top-level of a display hierarchy.
 
         :return: A :class:`lst` of concepts.
         """
         #Collections are not used in Heritagedata so get_top_concepts() equals get_top_display()
-        return self.get_top_display()
+        return self.get_top_display(**kwargs)
 
-    def get_top_display(self):
+    def get_top_display(self, **kwargs):
         """  Returns all concepts or collections that form the top-level of a display hierarchy.
         :return: A :class:`lst` of concepts and collections.
         """
-        params = {'schemeURI': self.scheme_uri}
-        return self._get_items("getTopConceptsForScheme", params)
+        params = {'schemeURI': self.scheme_uri, 'alias': True}
+        ret = self._get_items("getTopConceptsForScheme", params, **kwargs)
+        language = self._get_language(**kwargs)
+        sort = self._get_sort(**kwargs)
+        sort_order = self._get_sort_order(**kwargs)
+        return self._sort(ret, sort, language, sort_order == 'desc')
 
-    def get_children_display(self, id):
+    def get_children_display(self, id, **kwargs):
         """ Return a list of concepts or collections that should be displayed under this concept or collection.
 
         :param str id: A concept or collection id.
         :returns: A :class:`lst` of concepts and collections.
         """
-        params = {'conceptURI': self.scheme_uri + "/concepts/" + id}
-        return self._get_items("getConceptRelations", params)
+        params = {'conceptURI': self.scheme_uri + "/concepts/" + id, 'alias': True}
+        ret = self._get_items("getConceptRelations", params, **kwargs)
+        language = self._get_language(**kwargs)
+        sort = self._get_sort(**kwargs)
+        sort_order = self._get_sort_order(**kwargs)
+        return self._sort(ret, sort, language, sort_order == 'desc')
 
     def expand(self, id):
         """ Expand a concept or collection to all it's narrower concepts.
@@ -218,7 +254,7 @@ class HeritagedataProvider(VocabularyProvider):
     def _get_children(self, id, all=False):
         #If all=True this method works recursive
         request = self.service_scheme_uri + "/getConceptRelations"
-        res = requests.get(request, params={'conceptURI': self.scheme_uri + "/concepts/" + id})
+        res = self.session.get(request, params={'conceptURI': self.scheme_uri + "/concepts/" + id})
         res.encoding = 'utf-8'
         result = res.json()
         answer = []
@@ -232,7 +268,7 @@ class HeritagedataProvider(VocabularyProvider):
                         answer.extend(child_list)
         return answer
 
-    def _get_items(self, service, params):
+    def _get_items(self, service, params, **kwargs):
         # send request to Heritagedata
         """ Returns the results of a service method to a :class:`lst` of concepts (and collections).
             The return :class:`lst`  can be empty.
@@ -248,11 +284,11 @@ class HeritagedataProvider(VocabularyProvider):
 
         request = self.service_scheme_uri + "/" + service
         try:
-            res = requests.get(request, params=params)
+            res = self.session.get(request, params=params)
         except ConnectionError as e:
-            raise ProviderUnavailableException("Request could not be executed - Request: %s - Params: %s" % (request, params))
+            raise ProviderUnavailableException(f"Request could not be executed - Request: {request} - Params: {params}")
         if res.status_code == 404:
-            raise ProviderUnavailableException("Service not found (status_code 404) - Request: %s - Params: %s" % (request, params))
+            raise ProviderUnavailableException(f"Service not found (status_code 404) - Request: {request} - Params: {params}")
         res.encoding = 'utf-8'
         result = res.json()
         d = {}
@@ -267,22 +303,31 @@ class HeritagedataProvider(VocabularyProvider):
             property = None
             if 'property' in r.keys():
                 property = r['property']
-            if not service == 'getConceptRelations' or property == str(SKOS.narrower):
+            if not service == 'getConceptRelations' or property == "skos:narrower":
                 item = {
-                'id': _split_uri(uri, 1),
-                'uri': uri,
-                'type': 'concept',
-                'label': label,
-                'lang': language
+                    'id': _split_uri(uri, 1),
+                    'uri': uri,
+                    'type': 'concept',
+                    'label': label,
+                    'lang': language
                 }
-            if uri not in d:
-                d[uri] = item
-            if tags.tag(d[uri]['lang']).format == tags.tag(self._get_language()).format:
-                pass
-            elif tags.tag(item['lang']).format == tags.tag(self._get_language()).format:
-                d[uri] = item
-            elif tags.tag(item['lang']).language and (tags.tag(item['lang']).language.format == tags.tag(self._get_language()).language.format):
-                d[uri] = item
-            elif tags.tag(item['lang']).format == 'en':
-                d[uri] = item
+                if uri not in d:
+                    d[uri] = item
+                if tags.tag(d[uri]['lang']).format == tags.tag(self._get_language(**kwargs)).format:
+                    pass
+                elif tags.tag(item['lang']).format == tags.tag(self._get_language(**kwargs)).format:
+                    d[uri] = item
+                elif tags.tag(item['lang']).language and (tags.tag(item['lang']).language.format == tags.tag(self._get_language(**kwargs)).language.format):
+                    d[uri] = item
+                elif tags.tag(item['lang']).format == 'en':
+                    d[uri] = item
         return list(d.values())
+
+    def _sort(self, items, sort, language='en', reverse=False):
+        if sort is None:
+            sort = 'id'
+        if sort == 'sortlabel':
+            sort='label'
+        items.sort(key=lambda item: item[sort], reverse=reverse)
+        return items
+
