@@ -1,55 +1,52 @@
-# -*- coding: utf-8 -*-
 '''
 Utility functions for :mod:`skosprovider_heritagedata`.
 '''
 
-import sys
-
-try:
-    from urllib2 import URLError
-except ImportError:
-    from urllib.error import URLError
-
-from skosprovider.skos import (
-    Concept,
-    Label,
-    Note,
-    ConceptScheme)
-from skosprovider.exceptions import ProviderUnavailableException
-
 import logging
-log = logging.getLogger(__name__)
-
-PY3 = sys.version_info[0] == 3
-
-if PY3:  # pragma: no cover
-    binary_type = bytes
-else:  # pragma: no cover
-    binary_type = str
 
 import rdflib
+import requests
+from rdflib.namespace import DCTERMS
+from rdflib.namespace import RDF
+from rdflib.namespace import RDFS
+from rdflib.namespace import SKOS
 from rdflib.term import URIRef
-from rdflib.namespace import RDF, SKOS, DC, RDFS
+from skosprovider.exceptions import ProviderUnavailableException
+from skosprovider.skos import Concept
+from skosprovider.skos import ConceptScheme
+from skosprovider.skos import Label
+from skosprovider.skos import Note
 
+log = logging.getLogger(__name__)
 PROV = rdflib.Namespace('http://www.w3.org/ns/prov#')
 
-def conceptscheme_from_uri(conceptscheme_uri):
+def conceptscheme_from_uri(conceptscheme_uri, **kwargs):
     '''
     Read a SKOS Conceptscheme from a :term:`URI`
 
     :param string conceptscheme_uri: URI of the conceptscheme.
     :rtype: skosprovider.skos.ConceptScheme
     '''
-    graph = uri_to_graph('%s.rdf' % (conceptscheme_uri))
+    s = kwargs.get('session', requests.Session())
+    graph = uri_to_graph('%s.rdf' % (conceptscheme_uri), session=s)
+
+    notes = []
+    labels = []
 
-    # get the conceptscheme
-    conceptscheme = ConceptScheme(conceptscheme_uri)
-    conceptscheme.notes = []
-    conceptscheme.labels = []
     if graph is not False:
         for s, p, o in graph.triples((URIRef(conceptscheme_uri), RDFS.label, None)):
             label = Label(o.toPython(), "prefLabel", 'en')
-            conceptscheme.labels.append(label)
+            labels.append(label)
+        for s, p, o in graph.triples((URIRef(conceptscheme_uri), DCTERMS.description, None)):
+            note = Note(o.toPython(), "scopeNote", 'en')
+            notes.append(note)
+
+    # get the conceptscheme
+    conceptscheme = ConceptScheme(
+        conceptscheme_uri,
+        labels=labels,
+        notes=notes
+    )
     return conceptscheme
 
 
@@ -63,16 +60,21 @@ def things_from_graph(graph, concept_scheme):
     :rtype: :class:`list`
     '''
     clist = []
+    valid_label_types = Label.valid_types[:]
+    valid_label_types.remove('sortLabel')
     for sub, pred, obj in graph.triples((None, RDF.type, SKOS.Concept)):
         uri = str(sub)
-        con = Concept(_split_uri(uri, 1), uri=uri)
-        con.broader = _create_from_subject_predicate(graph, sub, SKOS.broader)
-        con.narrower = _create_from_subject_predicate(graph, sub, SKOS.narrower)
-        con.related = _create_from_subject_predicate(graph, sub, SKOS.related)
-        con.labels = _create_from_subject_typelist(graph, sub, Label.valid_types)
-        con.notes = _create_from_subject_typelist(graph, sub, Note.valid_types)
-        con.subordinate_arrays = []
-        con.concept_scheme = concept_scheme
+        con = Concept(
+            id=_split_uri(uri, 1),
+            uri=uri,
+            concept_scheme = concept_scheme,
+            labels = _create_from_subject_typelist(graph, sub, valid_label_types),
+            notes = _create_from_subject_typelist(graph, sub, Note.valid_types),
+            broader = _create_from_subject_predicate(graph, sub, SKOS.broader),
+            narrower = _create_from_subject_predicate(graph, sub, SKOS.narrower),
+            related = _create_from_subject_predicate(graph, sub, SKOS.related),
+            subordinate_arrays = []
+        )
         clist.append(con)
 
         # at this moment, Heritagedata does not support SKOS.Collection
@@ -134,28 +136,32 @@ def _split_uri(uri, index):
     return uri.strip('/').rsplit('/', 1)[index]
 
 
-def uri_to_graph(uri):
+def uri_to_graph(uri, **kwargs):
     '''
     :param string uri: :term:`URI` where the RDF data can be found.
     :rtype: rdflib.Graph
     :raises skosprovider.exceptions.ProviderUnavailableException: if the 
         heritagedata.org services are down
     '''
+    s = kwargs.get('session', requests.Session())
     graph = rdflib.Graph()
     try:
-        graph.parse(uri)
-        if len(graph) == 0:
-            return False
-        return graph
-    except URLError as e:
+        res = s.get(uri)
+    except requests.ConnectionError as e:
         raise ProviderUnavailableException("URI not available: %s" % uri)
+    if res.status_code == 404:
+        return False
+    graph.parse(data=res.content)
+    # heritagedata.org returns a empy page/graph when a resource does not exists
+    # (statsu_code 200). For this reason we return False if the graph is empty
+    if len(graph) == 0:
+        return False
+    return graph
 
 
 def text_(s, encoding='latin-1', errors='strict'):
-    """ If ``s`` is an instance of ``binary_type``, return
+    """ If ``s`` is an instance of ``bytes``, return
     ``s.decode(encoding, errors)``, otherwise return ``s``"""
-    if isinstance(s, binary_type):
+    if isinstance(s, bytes):
         return s.decode(encoding, errors)
     return s
-
-
